/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.UserAdminViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.MessageDialog");
dojo.require("com.ibm.rational.report.web.core.app.view.admin.EditContributersDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ProjectClient");
dojo.require("com.ibm.rational.report.web.core.client.internal.UserClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.FilteringTable");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.PaginationControl");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var AdminMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ProjectClient			= com.ibm.rational.report.web.core.client.internal.ProjectClient;
var UserClient				= com.ibm.rational.report.web.core.client.internal.UserClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PaginationControl	  	= com.ibm.rational.report.web.core.app.internal.common.PaginationControl;
var InternalMessages 		= com.ibm.rational.report.web.core.app.internal;

// Most Recently used values
var MRU 					= new Object();
MRU.users					= new Array();
var startIndex = 0;
var pageSize = 0;
var ADMINS					= new Array();

/**
 * Displays the Manage users views.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.UserAdminViewer",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/UserAdminViewer.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
	},
	
	postCreate: function() {
		Util.removeChildren(this.pageTitle);
		this.pageTitle.appendChild(document.createTextNode(AdminMessages.USER_ADMIN_VIEWER_PAGE_TITLE));

		this._msgFormatter = new MessageFormatter(this.messageArea);
		this._projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
					
		this.toolbar = new dijit.Toolbar({layoutAlign: "top", style: "background : transparent; border: none"});
		
		this.deleteButton = new dijit.form.Button({title:CommonMessages.DELETE_USERS_TOOLTIP, iconClass: "RemoveButtonIcon", label: CommonMessages.DELETE_USERS_TOOLTIP, showLabel: false,
			onClick: dojo.hitch(this, this._displayConfirmationDialog) }); 
		this.newButton = new dijit.form.Button({title:CommonMessages.ADD_USERS_TOOLTIP, iconClass: "AddButtonIcon", label: CommonMessages.ADD_USERS_TOOLTIP, showLabel: false,
			onClick: dojo.hitch(this, this._editContributors)});
				
		this.deleteButton.disabled = true;
		this.deleteButton._setStateClass();
		
		this.toolbar.addChild(this.newButton); 
		this.toolbar.addChild(this.deleteButton);
		
		this.imageButtons.appendChild(this.toolbar.domNode);

		var args = new Object();
		args.viewer = this;
		this.paginationControl = new PaginationControl(args);
	
		this.paginationButtons.appendChild(this.paginationControl.domNode);

		var chk = document.createElement("input");
		chk.type = "checkbox";
		this._tableColumns = [
			{field: "Select", sortType: "__markup__", label:chk,  width: "12px"},
			{field: "UserId",  label: AdminMessages.USER_ADMIN_TABLE_USER_ID_LABEL},
			{field: "FirstName",  label: AdminMessages.USER_ADMIN_TABLE_FIRST_NAME_LABEL},
			{field: "LastName", label: AdminMessages.USER_ADMIN_TABLE_LAST_NAME_LABEL},
			{field: "ScreenName", label: AdminMessages.USER_ADMIN_TABLE_SCREEN_NAME_LABEL},
			{field: "EmailAddress",label: AdminMessages.USER_ADMIN_TABLE_EMAIL_ADDRESS_LABEL}
		];
		
		this._table = new com.ibm.rational.report.web.core.app.internal.common.FilteringTable(
				{valueField: "rowId", noSelectField: "Select"}, this.userTable);
			
		for (var i = 0; i < this._tableColumns.length; i++) {
			this._table.columns.push(this._table.createMetaData(this._tableColumns[i]));
		}
		
		dojo.connect(this._table, "onGroupSelect", this, "_onGroupSelect");
		
		this.displayContent();
	},
	
	getAddUserDialog: function() {
		return new com.ibm.rational.report.web.core.app.view.admin.EditContributersDialog({controller: this._controller, viewer: this});
	},
	
	_setSelectToolTips: function() {
		var inputs = this.userTable.tBodies[0].getElementsByTagName("input");
		dojo.forEach(inputs, function(input) {
			input.title = (input.checked)?  CommonMessages.DESELECT_USER_TOOLTIP :
				CommonMessages.SELECT_USER_TOOLTIP;		
		});
	},
	
	_onGroupSelect : function() {
		var rows = this._table.getGroupSelectedRows();
		this.deleteButton.disabled = (rows <= 0);
		this.deleteButton._setStateClass();
		this._setSelectToolTips();
	},
	
	_editContributors : function() {
		var self = this;
		var editContributersDialog = new com.ibm.rational.report.web.core.app.view.admin.EditContributersDialog(
			{controller: this._controller, viewer: this});
		dojo.connect(editContributersDialog.okButton, "onclick", editContributersDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				editContributersDialog.addContributors();
				Util.removeChildren(self.messageArea);
				self._msgFormatter.displayLoadingMessage(InternalMessages.LOADING_PROGRESS_MSG);
				editContributersDialog.close();
			}
		);	
		dojo.connect(editContributersDialog.cancelButton, "onclick", editContributersDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				editContributersDialog.close();
			}
		);
	},
	
	_grantAccessToAllUsers: function(e) {
		var srh = new ServiceResponseHandler(this, "_addAllUsers", "_getUsersError");				
		UserClient.getUsers(srh, {namePattern: "*"});
	},
	
	_addAllUsers: function(users) {
		this.messageArea.appendChild(document.createTextNode(AdminMessages.USER_ADMIN_GRANTING_WEB_SITE_ACCESS_TO_ALL_USERS));
		
		var userNames = new Array();		
		for (var i = 0; i < users.length; i++) {
			var user = users[i];
			userNames[i] = user.name;
		}
		var srh = new ServiceResponseHandler(this, "_postAddAllUserSuccess", "_addAllUsersError");		
		UserClient.postNewContributors(srh, {names: userNames});			
	},
	
	_postAddAllUserSuccess: function(contributors) {
		var contributorIds = new Array();
		for (var i = 0; i < contributors.length; i++) {
			var contributor = contributors[i];
			contributorIds[i] = contributor.itemId;
		}
				
		var srh = new ServiceResponseHandler(this, "_addAllUsersSuccess", "_addAllUsersError");
		ProjectClient.postAddProjectMembers(srh, {projectAreaId: this._projectArea.itemId, contributorIds: contributorIds});
	},
	
	_addAllUsersSuccess: function() {
		this.displayContent();
		this._msgFormatter.displaySuccessMessage(ServiceabilityMessages.GRANTED_ALL_USERS_WEBSITE_ACCESS);		
	},
		
	_addAllUsersError: function(error) {
		this.displayContent();
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_GRANT_SOME_USERS_WEBSITE_ACCESS, error);
	},
	
	displayContent: function() {
		if (ADMINS.length > 0) {
			this._displayUsersStart();
		}
		else {	
			this._msgFormatter.displayLoadingMessage(AdminMessages.USER_ADMIN_RETRIEVING_USERS_FROM_SERVER);
			var srh = new ServiceResponseHandler(this, "_getAdminUsers", "_getAdminUsersError");				
			UserClient.getAdminUsers(srh);
		}			
	},
	
	_getAdminUsers: function(result) {
		if (result != null) {
			for (var i = 0; i < result.length; i++) {
				ADMINS[result[i].name] = 1;
			}
		}
		this._displayUsersStart();
	},
	
	_getAdminUsersError: function() {
		this._displayUsersStart();
	},
		
	_displayUsersStart: function() {
		startIndex = this.paginationControl.getStartIndex();
		pageSize = this.paginationControl.getPageSize();
		var name = dojo.trim("*");
		var srh = new ServiceResponseHandler(this, "_displayUsers", "_getUsersError");				
		UserClient.getAllUserAttributes(srh, {projectAreaId: this._projectArea.itemId, namePattern: name, contributersOnly: "true", startIndex: startIndex , length : pageSize});			
	},
	
	_displayUsers: function(users) {
		this._msgFormatter.displayNoMessage();
		this._table.store.clearData();
		this.paginationControl.setNextLink(users != null && users.length > 0);
		this.paginationControl.setPreviousLink(startIndex > 1);
		if (users != null) {
			// update paginationControl as per the results
			this.paginationControl.setPageInfo(startIndex, startIndex + users.length - 1);
						
			var sortedUsers = Util.sort(users);
			for (var i = 0; i < sortedUsers.length; i++) {
				var user = sortedUsers[i];
				var chk = document.createElement("input");
				chk.type = "checkbox";
				chk.id = user.name;
				if (ADMINS[user.name] == 1) {
					// disable the checkbox for an admin user
					chk.disabled = true;
				}
				chk.title = CommonMessages.SELECT_USER_TOOLTIP;
					this._table.store.addData({
						rowId: i,
						UserId: user.name,
						FirstName: user.firstName,
						LastName: user.lastName,
						ScreenName: user.screenName,
						EmailAddress: user.email,
						Select: chk										
					});
			}				
		} else {
			this.paginationControl.setNextLink(false);
		}
	},
	
	_getUsersError: function(error) {
		dojox.data.dom.removeChildren(this.messageArea);
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETREIVE_USERS, error);
	},
	
	_denyAccessToSelectedUsers: function(e) {
		Util.removeChildren(this.messageArea);
		this._msgFormatter.displayLoadingMessage(InternalMessages.LOADING_PROGRESS_MSG);
		var userNames = new Array();		
		var inputs = this.userTable.getElementsByTagName("input");
		var i = 0;
		dojo.forEach(inputs, function(input) {
			if (input.checked && input.id != "") {
				userNames[i++] = input.id;
			}
		});
		if (userNames.length != 0) {
			var srh = new ServiceResponseHandler(this, "_deleteContributorsSuccess", "_deleteContributorsError");
			UserClient.postDeleteContributors(srh, {
				names: userNames
			});
		}
	},
	
	_deleteContributorsSuccess : function() {
		this.displayContent();
		this._msgFormatter.displaySuccessMessage(ServiceabilityMessages.DENIED_SELECTED_USERS_WEBSITE_ACCESS);
	},
	
	_deleteContributorsError: function(error) {
		this.displayContent();
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_DENY_SELECTED_USERS_ACCESS_TO_WEBSITE, error);
	},
	
	_postAddUsersSuccess: function(contributors) {
		var contributorIds = new Array();
		for (var i = 0; i < contributors.length; i++) {
			var contributor = contributors[i];
			contributorIds[i] = contributor.itemId;
		}
				
		var srh = new ServiceResponseHandler(this, "_addUsersSuccess", "_addUsersError");
		ProjectClient.postAddProjectMembers(srh, {projectAreaId: this._projectArea.itemId, contributorIds: contributorIds});
	},
	
	_addUsersSuccess: function() {
		this._msgFormatter.displaySuccessMessage(ServiceabilityMessages.GRANTED_SELECTED_USERS_WEBSITE_ACCESS);		
	},
	
	_addUsersError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_GRANT_SELECTED_USERS_ACCESS_TO_WEBSITE, error);
	},
	
	_refreshSuccess: function() {
		dojox.data.dom.removeChildren(this.messageArea);
		this.refreshButton.disabled = false;
		this._msgFormatter.displaySuccessMessage(ServiceabilityMessages.RETREIVED_RECENTLY_ADDED_USERS);		
	},
	
	_refreshError: function(error) {
		dojox.data.dom.removeChildren(this.messageArea);
		this.refreshButton.disabled = false;
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETREIVE_RECENTLY_ADDED_USERS, error);
	},
	
	_displayConfirmationDialog: function() {
		var self = this;
		var confirmationDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.CONFIRMATION, prompt: CommonMessages.CONFIRMATION_DELETE_USERS, promptType: "Warning", buttonType: "YesNo"});
		dojo.connect(confirmationDialog.leftButton, "onclick", confirmationDialog.dialog, function (e) {
				dojo.stopEvent(e);
				self._denyAccessToSelectedUsers();
				self.deleteButton.disabled = true;
				self.deleteButton._setStateClass();
				confirmationDialog.close();
		});	
		dojo.connect(confirmationDialog.rightButton, "onclick", confirmationDialog.dialog, function (e) {
				dojo.stopEvent(e);
				confirmationDialog.close();
		});
	},
	
	clearMessageArea: function(){
		Util.removeChildren(this.messageArea);
	}
	
});

})();